config = {
    'name': 'remote-pkg',
    'macros': {
        'dont_replace': '$pkgdatadir/blah',
        'replace_by_second_project': 'not-replaced',
        'to_be_replaced': 'value-to-be-replaced',
    }
}
